@@ -50,12 +50,6 @@ pp_printer_dispose (GObject *object)
   G_OBJECT_CLASS (pp_printer_parent_class)->dispose (object);
 }
 
-static void
-pp_printer_finalize (GObject *object)
-{
-  G_OBJECT_CLASS (pp_printer_parent_class)->finalize (object);
-}
-
 static void
 pp_printer_get_property (GObject    *object,
                          guint       property_id,
@@ -104,7 +98,6 @@ pp_printer_class_init (PpPrinterClass *klass)
   gobject_class->set_property = pp_printer_set_property;
   gobject_class->get_property = pp_printer_get_property;
   gobject_class->dispose = pp_printer_dispose;
-  gobject_class->finalize = pp_printer_finalize;
 
   g_object_class_install_property (gobject_class, PROP_NAME,
     g_param_spec_string ("printer-name",
@@ -155,11 +148,11 @@ printer_rename_dbus_cb (GObject      *source_object,
                         GAsyncResult *res,
                         gpointer      user_data)
 {
-  PpPrinter        *self;
-  GVariant         *output;
-  gboolean          result = FALSE;
-  g_autoptr(GError) error = NULL;
-  GTask            *task = user_data;
+  PpPrinter          *self;
+  g_autoptr(GVariant) output = NULL;
+  gboolean            result = FALSE;
+  g_autoptr(GError)   error = NULL;
+  GTask              *task = user_data;
 
   output = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object),
                                           res,
@@ -186,8 +179,6 @@ printer_rename_dbus_cb (GObject      *source_object,
         }
 
       g_task_return_boolean (task, result);
-
-      g_variant_unref (output);
     }
   else
     {
@@ -424,10 +415,10 @@ pp_printer_delete_dbus_cb (GObject      *source_object,
                            GAsyncResult *res,
                            gpointer      user_data)
 {
-  GVariant         *output;
-  gboolean          result = FALSE;
-  g_autoptr(GError) error = NULL;
-  GTask            *task = user_data;
+  g_autoptr(GVariant) output = NULL;
+  gboolean            result = FALSE;
+  g_autoptr(GError)   error = NULL;
+  GTask              *task = user_data;
 
   output = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object),
                                           res,
@@ -448,8 +439,6 @@ pp_printer_delete_dbus_cb (GObject      *source_object,
         result = TRUE;
 
       g_task_return_boolean (task, result);
-
-      g_variant_unref (output);
     }
   else
     {
