@@ -202,12 +202,10 @@ cc_carousel_find_item (CcCarousel    *self,
 }
 
 static void
-on_item_toggled (CcCarouselItem *item,
+on_item_toggled (CcCarousel     *self,
                  GdkEvent       *event,
-                 gpointer        user_data)
+                 CcCarouselItem *item)
 {
-        CcCarousel *self = CC_CAROUSEL (user_data);
-
         cc_carousel_select_item (self, item);
 }
 
@@ -311,7 +309,7 @@ cc_carousel_add (GtkContainer *container,
         CC_CAROUSEL_ITEM (widget)->page = get_last_page_number (self);
         if (self->selected_item != NULL)
                 gtk_radio_button_join_group (GTK_RADIO_BUTTON (widget), GTK_RADIO_BUTTON (self->selected_item));
-        g_signal_connect (widget, "button-press-event", G_CALLBACK (on_item_toggled), self);
+        g_signal_connect_object (widget, "button-press-event", G_CALLBACK (on_item_toggled), self, G_CONNECT_SWAPPED);
 
         last_box_is_full = ((g_list_length (self->children) - 1) % ITEMS_PER_PAGE == 0);
         if (last_box_is_full) {
@@ -429,8 +427,8 @@ cc_carousel_init (CcCarousel *self)
 
         g_object_unref (provider);
 
-        g_signal_connect_swapped (self->stack, "size-allocate", G_CALLBACK (on_size_allocate), self);
-        g_signal_connect_swapped (self->stack, "notify::transition-running", G_CALLBACK (on_transition_running), self);
+        g_signal_connect_object (self->stack, "size-allocate", G_CALLBACK (on_size_allocate), self, G_CONNECT_SWAPPED);
+        g_signal_connect_object (self->stack, "notify::transition-running", G_CALLBACK (on_transition_running), self, G_CONNECT_SWAPPED);
 }
 
 guint
